@@ -1,12 +1,11 @@
 package AI::Prolog::Term;
-$REVISION = '$Id: Term.pm,v 1.6 2005/02/28 02:32:11 ovid Exp $';
+$REVISION = '$Id: Term.pm,v 1.8 2005/06/20 02:03:02 ovid Exp $';
 
-$VERSION = '0.05';
+$VERSION = '0.06';
 use strict;
 use warnings;
 
 use aliased 'AI::Prolog::Term::Cut';
-use aliased 'AI::Prolog::Term::Number';
 use aliased 'AI::Prolog::Parser';
 
 use aliased 'Hash::AsObject';
@@ -54,14 +53,12 @@ sub new {
     return $class->_new_var unless @_;
     if (2 == @_) { # more common (performance)
         return _new_from_functor_and_arity($class, @_) unless 'ARRAY' eq CORE::ref $_[1];
-        # XXX unused?
-        return _new_from_functor_and_array($class, @_) if     'ARRAY' eq CORE::ref $_[1];
     }
     elsif (1 == @_) {
         my $arg = shift;
         return _new_with_id($class, $arg)     if ! CORE::ref $arg && $arg =~ /^[[:digit:]]+$/;
         return _new_from_string($class, $arg) if ! CORE::ref $arg;
-        return _new_from_parser($class, $arg) if   CORE::ref $arg && $arg->isa(Parser);
+        return $arg->_term($class)            if   CORE::ref $arg && $arg->isa(Parser);
     }
     require Carp;
     Carp::croak("Unknown arguments to Term->new");
@@ -69,8 +66,7 @@ sub new {
 
 sub _new_from_string {
     my ($class, $string) = @_;
-    my $parsed = Parser->new($string);
-    return _new_from_parser($class, $parsed);
+    my $parsed = Parser->new($string)->_term($class);;
 }
 
 sub _new_var {
@@ -103,21 +99,6 @@ sub _new_with_id {
     } => $class;
 }
 
-sub _new_from_functor_and_array {
-    my ($class, $functor, $arrayref) = @_;
-    bless {
-        functor => $functor,
-        arity   => scalar @$arrayref,
-        args    => $arrayref,
-        # if bound is false, $self is a reference to a free variable
-        bound   => 1,
-        varid   => undef, # XXX ??
-        # if bound and deref are both true, $self is a reference to a ref
-        deref   => 0,
-        ref     => undef,
-    } => $class;
-}
-
 sub _new_from_functor_and_arity {
     my ($class, $functor, $arity) = @_;
     bless {
@@ -133,131 +114,6 @@ sub _new_from_functor_and_arity {
     } => $class;
 }
 
-# This constructor is the simplest way to construct a term.  The term is given
-# in standard notation.
-# Example: my $term = Term->new(Parser->new("p(1,a(X,b))"));
-sub _new_from_parser {
-    my ($class, $parser) = @_;
-    my $self = bless {
-        functor => undef,
-        arity   => 0,
-        args    => [],
-        # if bound is false, $self is a reference to a free variable
-        bound   => 0,
-        varid   => undef, # XXX ??
-        # if bound and deref are both true, $self is a reference to a ref
-        deref   => 0,
-        ref     => undef,
-    } => $class;
-    my $ts   = [];
-    my $i    = 0;
-
-    $parser->skipspace; # otherwise we crash when we hit leading
-                        # spaces
-    if ($parser->current =~ /^[[:lower:]'"]$/) {
-        $self->{functor} = $parser->getname;
-        $self->{bound}   = 1;
-        $self->{deref}   = 0;
-
-        if ('(' eq $parser->current) {
-            $parser->advance;
-            $parser->skipspace;
-            $ts->[$i++] = $self->new($parser);
-            $parser->skipspace;
-
-            while (',' eq $parser->current) {
-                $parser->advance;
-                $parser->skipspace;
-                $ts->[$i++] = $self->new($parser);
-                $parser->skipspace;
-            }
-
-            if (')' ne $parser->current) {
-                $parser->parseerror("Expecting: ')'.  Got (@{[$parser->current]})");
-            }
-
-            $parser->advance;
-            $self->{args} = [];
-
-            $self->{args}[$_] = $ts->[$_] for 0 .. ($i -1);
-            $self->{arity} = $i;
-        }
-        else {
-           $self->{arity} = 0;
-        }
-    }
-    elsif ($parser->current =~ /^[[:upper:]]$/) {
-        $self->{bound}     = 1;
-        $self->{deref}     = 1;
-        my ($ref, $string) = $parser->getvar;
-        $self->{ref}       = $ref;
-        $self->{varname}   = $string;
-    }
-    elsif ($parser->current =~ /^[-.[:digit:]]$/) {
-        return Number->new($parser->getnum);
-    }
-    elsif ('[' eq $parser->current) {
-        $parser->advance;
-
-        if (']' eq $parser->current) {
-            $parser->advance;
-            $self->{functor} = NULL;
-            $self->{arity}   = 0;
-            $self->{bound}   = 1;
-            $self->{deref}   = 0;
-        }
-        else {
-            $parser->skipspace;
-            $ts->[$i++] = $self->new($parser);
-            $parser->skipspace;
-
-            while (',' eq $parser->current) {
-                $parser->advance;
-                $parser->skipspace;
-                $ts->[$i++] = $self->new($parser);
-                $parser->skipspace;
-            }
-
-            if ('|' eq $parser->current) {
-                $parser->advance;
-                $parser->skipspace;
-                $ts->[$i++] = $self->new($parser);
-                $parser->skipspace;
-            }
-            else {
-                $ts->[$i++] = $self->new(NULL, 0);
-            }
-
-            if (']' ne $parser->current) {
-                $parser->parseerror("Expecting ']'");
-            }
-
-            $parser->advance;
-            $self->{bound}   = 1;
-            $self->{deref}   = 0;
-            $self->{functor} = "cons";
-            $self->{arity}   = 2;
-            $self->{args}    = [];
-            for (my $j = $i - 2; $j > 0; $j--) {
-                my $term = $self->new("cons", 2);
-                $term->setarg(0, $ts->[$j]);
-                $term->setarg(1, $ts->[$j+1]);
-                $ts->[$j] = $term;
-            }
-            $self->{args}[0] = $ts->[0];
-            $self->{args}[1] = $ts->[1];
-        }
-    }
-    elsif ('!' eq $parser->current) {
-        $parser->advance;
-        return $self->CUT;
-    }
-    else {
-        $parser->parseerror("Term should begin with a letter, a digit, or '[', not a @{[$parser->current]}");
-    }
-    return $self;
-}
-
 sub varnum  { $VARNUM          } # class method
 sub functor { shift->{functor} }
 sub arity   { shift->{arity}   }
